@@ -21,10 +21,6 @@
  * 02111-1307, USA.
  */
 
-#ifdef GDK_DISABLE_DEPRECATED
-#undef GDK_DISABLE_DEPRECATED
-#endif
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -42,14 +38,16 @@
 
 #include "applier.h"
 
-#define MONITOR_CONTENTS_X 0
+#define MONITOR_CONTENTS_X 0 
 #define MONITOR_CONTENTS_Y 0
-#define MONITOR_CONTENTS_WIDTH 51
-#define MONITOR_CONTENTS_HEIGHT 38
+#define MONITOR_CONTENTS_DEFAULT_WIDTH 64
+#define MONITOR_CONTENTS_DEFAULT_HEIGHT 48
 
 enum {
 	PROP_0,
-	PROP_TYPE
+	PROP_TYPE,
+	PROP_PREVIEW_WIDTH,
+	PROP_PREVIEW_HEIGHT
 };
 
 struct _BGApplierPrivate 
@@ -64,10 +62,6 @@ struct _BGApplierPrivate
 
 	GdkPixbuf          *wallpaper_pixbuf;      /* The "raw" wallpaper pixbuf */
 
-	gboolean            nautilus_running;      /* TRUE iff nautilus is
-						    * running, in which case we
-						    * block the renderer */
-
 	BGApplierType         type;                  /* Whether we render to the
 						    * root or the preview */
 
@@ -133,7 +127,9 @@ static void bg_applier_finalize      (GObject           *object);
 
 static void run_render_pipeline      (BGApplier           *bg_applier, 
 				      const BGPreferences *prefs);
-static void draw_disabled_message    (GtkWidget         *widget);
+static void draw_disabled_message    (GtkWidget         *widget,
+				      const guint width,
+				      const guint height);
 
 static void render_background        (BGApplier           *bg_applier,
 				      const BGPreferences *prefs);
@@ -182,10 +178,13 @@ static gboolean is_nautilus_running  (void);
 
 static gboolean cleanup_cb           (BGApplier *bg_applier);
 
-guint
+static void preview_realized_cb      (GtkWidget *preview,
+				      BGApplier *bg_applier);
+
+GType
 bg_applier_get_type (void)
 {
-	static guint bg_applier_type = 0;
+	static GType bg_applier_type = 0;
 
 	if (!bg_applier_type) {
 		static GTypeInfo bg_applier_info = {
@@ -217,8 +216,9 @@ bg_applier_init (BGApplier *bg_applier, BGApplierClass *class)
 	bg_applier->p->last_prefs       = NULL;
 	bg_applier->p->pixbuf           = NULL;
 	bg_applier->p->wallpaper_pixbuf = NULL;
-	bg_applier->p->nautilus_running = is_nautilus_running ();
 	bg_applier->p->timeout          = 0;
+	bg_applier->p->render_geom.width = -1;
+	bg_applier->p->render_geom.height = -1;
 }
 
 static void
@@ -241,6 +241,22 @@ bg_applier_class_init (BGApplierClass *class)
 				   0, 1, 0,
 				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property
+		(object_class, PROP_PREVIEW_WIDTH,
+		 g_param_spec_uint ("preview_width",
+			 	    _("Preview Width"),
+				    _("Width if applier is a preview: Defaults to 64."),
+				    1, 65535, MONITOR_CONTENTS_DEFAULT_WIDTH,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property
+		(object_class, PROP_PREVIEW_HEIGHT,
+		 g_param_spec_uint ("preview_height",
+				    _("Preview Height"),
+				    _("Height if applier is a preview: Defaults to 48."),
+				    1, 65535, MONITOR_CONTENTS_DEFAULT_HEIGHT,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	parent_class = 
 		G_OBJECT_CLASS (g_type_class_ref (G_TYPE_OBJECT));
 }
@@ -277,8 +293,13 @@ bg_applier_set_prop (GObject *object, guint prop_id, const GValue *value, GParam
 		case BG_APPLIER_PREVIEW:
 			bg_applier->p->render_geom.x = MONITOR_CONTENTS_X;
 			bg_applier->p->render_geom.y = MONITOR_CONTENTS_Y;
-			bg_applier->p->render_geom.width = MONITOR_CONTENTS_WIDTH;
-			bg_applier->p->render_geom.height = MONITOR_CONTENTS_HEIGHT;
+			
+			if (bg_applier->p->render_geom.width == -1)
+			{
+				bg_applier->p->render_geom.width = MONITOR_CONTENTS_DEFAULT_WIDTH;
+				bg_applier->p->render_geom.height = MONITOR_CONTENTS_DEFAULT_HEIGHT;
+			}
+
 			break;
 
 		default:
@@ -288,6 +309,16 @@ bg_applier_set_prop (GObject *object, guint prop_id, const GValue *value, GParam
 
 		break;
 
+	case PROP_PREVIEW_WIDTH:
+		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
+			bg_applier->p->render_geom.width = g_value_get_uint (value);
+		break;
+
+	case PROP_PREVIEW_HEIGHT:
+		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
+			bg_applier->p->render_geom.height = g_value_get_uint (value);
+		break;
+		
 	default:
 		g_warning ("Bad property set");
 		break;
@@ -363,6 +394,22 @@ bg_applier_new (BGApplierType type)
 	return object;
 }
 
+GObject *
+bg_applier_new_at_size (BGApplierType type,
+			const guint width,
+			const guint height)
+{
+	GObject *object;
+
+	object = g_object_new (bg_applier_get_type (),
+			       "type", type,
+			       "preview_width", width,
+			       "preview_height", height,
+			       NULL);
+
+	return object;
+}
+
 void
 bg_applier_apply_prefs (BGApplier           *bg_applier, 
 			const BGPreferences *prefs)
@@ -374,12 +421,20 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 
 	new_prefs = BG_PREFERENCES (bg_preferences_clone (prefs));
 
-	if (bg_applier->p->type == BG_APPLIER_ROOT && bg_applier->p->nautilus_running)
+	if (new_prefs->wallpaper_type == WPTYPE_NONE)
+	{
+		new_prefs->wallpaper_enabled = FALSE;
+		new_prefs->wallpaper_type = WPTYPE_CENTERED;
+	}
+
+	if (bg_applier->p->type == BG_APPLIER_ROOT && is_nautilus_running ()) {
 		set_root_pixmap ((GdkPixmap *) -1);
+		return;
+	}
 
 	if (!new_prefs->enabled) {
 		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
-			draw_disabled_message (bg_applier_get_preview_widget (bg_applier));
+			draw_disabled_message (bg_applier_get_preview_widget (bg_applier), bg_applier->p->render_geom.width, bg_applier->p->render_geom.height);
 		return;
 	}
 
@@ -390,27 +445,20 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 		bg_applier->p->wallpaper_pixbuf = NULL;
 
 		if (new_prefs->wallpaper_enabled) {
-		        if (new_prefs->wallpaper_filename == NULL) {
-			        new_prefs->wallpaper_enabled = FALSE;
-			} else {
-			        g_return_if_fail (new_prefs->wallpaper_filename != NULL);
+			g_return_if_fail (new_prefs->wallpaper_filename != NULL);
 
-				bg_applier->p->wallpaper_pixbuf = 
-				  gdk_pixbuf_new_from_file (new_prefs->wallpaper_filename, NULL);
+			bg_applier->p->wallpaper_pixbuf = 
+				gdk_pixbuf_new_from_file (new_prefs->wallpaper_filename, NULL);
 
-				if (bg_applier->p->wallpaper_pixbuf == NULL) {
-				        g_warning (_("Could not load pixbuf \"%s\"; disabling wallpaper."),
-						   new_prefs->wallpaper_filename);
-					new_prefs->wallpaper_enabled = FALSE;
-				
-				}
-				else
-				{
-					if (bg_applier->p->timeout)
+			if (bg_applier->p->wallpaper_pixbuf == NULL) {
+				g_warning (_("Could not load pixbuf \"%s\"; disabling wallpaper."),
+					   new_prefs->wallpaper_filename);
+				new_prefs->wallpaper_enabled = FALSE;
+			}
+			else if (bg_applier->p->type == BG_APPLIER_ROOT) {
+				if (bg_applier->p->timeout)
 						g_source_remove (bg_applier->p->timeout);
-					bg_applier->p->timeout = g_timeout_add (30000, (GSourceFunc) cleanup_cb, bg_applier);
-
-				}
+				bg_applier->p->timeout = g_timeout_add (30000, (GSourceFunc) cleanup_cb, bg_applier);
 			}
 		}
 	}
@@ -443,15 +491,25 @@ bg_applier_render_color_p (const BGApplier *bg_applier, const BGPreferences *pre
 GtkWidget *
 bg_applier_get_preview_widget (BGApplier *bg_applier) 
 {
-  if (bg_applier->p->preview_widget == NULL) {
-    GdkPixmap *pixmap;
+	if (bg_applier->p->preview_widget == NULL)
+	{
+		GdkPixmap *pixmap;
 
-    /* fixme: What to do here? gdk does not export root_parent publicly (Lauris) */
-    pixmap = gdk_pixmap_new (GDK_ROOT_PARENT(), 51, 38, -1);
-    bg_applier->p->preview_widget = gtk_image_new_from_pixmap (pixmap, NULL);
-  }
+		pixmap = gdk_pixmap_new (gdk_get_default_root_window (),
+					 bg_applier->p->render_geom.width,
+					 bg_applier->p->render_geom.height,
+					 -1);
+		bg_applier->p->preview_widget = gtk_image_new_from_pixmap (pixmap, NULL);
 
-  return bg_applier->p->preview_widget;
+		/* We need to initialize the pixmap, but this
+		 * needs GCs, so we have to wait until realize. */
+		g_signal_connect (G_OBJECT (bg_applier->p->preview_widget),
+				  "realize",
+				  (GCallback) preview_realized_cb,
+				  bg_applier);
+	}
+
+	return bg_applier->p->preview_widget;
 }
 
 GdkPixbuf *
@@ -464,23 +522,19 @@ bg_applier_get_wallpaper_pixbuf (BGApplier *bg_applier)
 }
 
 static void
-draw_disabled_message (GtkWidget *widget)
+draw_disabled_message (GtkWidget *widget, const guint w, const guint h)
 {
 	GdkPixmap      *pixmap;
 	GdkColor        color;
 	PangoLayout    *layout;
 	PangoRectangle  extents;
 	GdkGC          *gc;
-	gint            x, y, w, h;
+	gint            x, y;
 	const char     *disabled_string = _("Disabled");
 
-	printf ("disabled\n");
-
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_IMAGE (widget));
 
-	w = MONITOR_CONTENTS_WIDTH;
-	h = MONITOR_CONTENTS_HEIGHT;
 	x = MONITOR_CONTENTS_X;
 	y = MONITOR_CONTENTS_Y;
 
@@ -513,7 +567,7 @@ draw_disabled_message (GtkWidget *widget)
 			 y + (h - extents.height) / 2 + extents.height / 2,
 			 layout);
 
-	gdk_gc_unref (gc);
+	g_object_unref (G_OBJECT (gc));
 	g_object_unref (G_OBJECT (layout));
 }
 
@@ -630,7 +684,7 @@ render_wallpaper (BGApplier *bg_applier, const BGPreferences *prefs)
 		if (bg_applier->p->wallpaper_pixbuf == NULL)
 			return;
 
-		gdk_window_get_size (GDK_ROOT_PARENT (), &tmp1, &tmp2);
+		gdk_drawable_get_size (gdk_get_default_root_window (), &tmp1, &tmp2);
 		virtual_geom.x = virtual_geom.y = 0;
 		virtual_geom.width = tmp1;
 		virtual_geom.height = tmp2;
@@ -719,7 +773,7 @@ render_to_screen (BGApplier *bg_applier, const BGPreferences *prefs)
 	g_return_if_fail (prefs != NULL);
 	g_return_if_fail (IS_BG_PREFERENCES (prefs));
 
-	gc = gdk_gc_new (GDK_ROOT_PARENT ());
+	gc = gdk_gc_new (gdk_get_default_root_window ());
 
 	if (bg_applier->p->pixbuf != NULL) {
 		if (bg_applier->p->pixbuf_render_geom.x != 0 ||
@@ -727,7 +781,15 @@ render_to_screen (BGApplier *bg_applier, const BGPreferences *prefs)
 		    bg_applier->p->pixbuf_render_geom.width != bg_applier->p->render_geom.width ||
 		    bg_applier->p->pixbuf_render_geom.height != bg_applier->p->render_geom.height)
 		{
-			gdk_color_alloc (gdk_window_get_colormap (GDK_ROOT_PARENT ()), prefs->color1);
+			gboolean success;
+
+#if 0
+			gdk_color_alloc (gdk_window_get_colormap (gdk_get_default_root_window ()), prefs->color1);
+#else
+			gdk_colormap_alloc_colors (gdk_drawable_get_colormap (gdk_get_default_root_window ()),
+						   prefs->color1, 1, FALSE, TRUE, &success);
+#endif
+
 			gdk_gc_set_foreground (gc, prefs->color1);
 			gdk_draw_rectangle (bg_applier->p->pixmap, gc, TRUE,
 					    bg_applier->p->render_geom.x,
@@ -748,18 +810,39 @@ render_to_screen (BGApplier *bg_applier, const BGPreferences *prefs)
 			 GDK_RGB_DITHER_MAX, 0, 0);
 	} else {
 		if (bg_applier->p->type == BG_APPLIER_ROOT) {
-			gdk_color_alloc (gdk_window_get_colormap (GDK_ROOT_PARENT()), prefs->color1);
-			gdk_window_set_background (GDK_ROOT_PARENT (), prefs->color1);
-			gdk_window_clear (GDK_ROOT_PARENT ());
+			gboolean success;
+
+#if 0
+			gdk_color_alloc (gdk_window_get_colormap (gdk_get_default_root_window()), prefs->color1);
+#else
+			gdk_colormap_alloc_colors (gdk_drawable_get_colormap (gdk_get_default_root_window ()),
+						   prefs->color1, 1, FALSE, TRUE, &success);
+#endif
+			gdk_window_set_background (gdk_get_default_root_window (), prefs->color1);
+			gdk_window_clear (gdk_get_default_root_window ());
 		}
 		else if (bg_applier->p->type == BG_APPLIER_PREVIEW) {
+			gboolean success;
+
+#if 0
 			gdk_color_alloc (gdk_window_get_colormap (bg_applier->p->preview_widget->window), prefs->color1);
-			gdk_gc_set_foreground (gc, prefs->color1);
-			gdk_draw_rectangle (bg_applier->p->pixmap, gc, TRUE,
-					    bg_applier->p->render_geom.x,
-					    bg_applier->p->render_geom.y, 
-					    bg_applier->p->render_geom.width,
-					    bg_applier->p->render_geom.height);
+#else
+			gdk_colormap_alloc_colors (gdk_drawable_get_colormap (gdk_get_default_root_window ()),
+						   prefs->color1, 1, FALSE, TRUE, &success);
+#endif
+
+			if (bg_applier->p->type == BG_APPLIER_PREVIEW) {
+				gdk_gc_set_foreground (gc, prefs->color1);
+				gdk_draw_rectangle (bg_applier->p->pixmap, gc, TRUE,
+						    bg_applier->p->render_geom.x,
+						    bg_applier->p->render_geom.y, 
+						    bg_applier->p->render_geom.width,
+						    bg_applier->p->render_geom.height);
+			}
+			else if (bg_applier->p->type == BG_APPLIER_ROOT) {
+				gdk_window_set_back_pixmap (gdk_get_default_root_window (), NULL, FALSE);
+				gdk_window_set_background (gdk_get_default_root_window (), prefs->color1);
+			}
 		}
 	}
 
@@ -806,8 +889,12 @@ create_pixmap (BGApplier *bg_applier, const BGPreferences *prefs)
 		if (!GTK_WIDGET_REALIZED (bg_applier->p->preview_widget))
 			gtk_widget_realize (bg_applier->p->preview_widget);
 
-		g_assert (gtk_image_get_storage_type (GTK_IMAGE(bg_applier->p->preview_widget)) == GTK_IMAGE_PIXMAP);
-		gtk_image_get_pixmap (GTK_IMAGE (bg_applier->p->preview_widget), &bg_applier->p->pixmap, NULL);
+#if 0
+		bg_applier->p->pixmap = GTK_PIXMAP (bg_applier->p->preview_widget)->pixmap;
+#else
+		if (!bg_applier->p->pixmap)
+			gtk_image_get_pixmap (GTK_IMAGE (bg_applier->p->preview_widget), &bg_applier->p->pixmap, NULL);
+#endif
 		bg_applier->p->pixmap_is_set = TRUE;
 		break;
 	}
@@ -918,8 +1005,6 @@ get_geometry (wallpaper_type_t  wallpaper_type,
 		src_geom->width = pwidth;
 		src_geom->height = pheight;
 		break;
-	case WPTYPE_EMBOSSED:
-	        g_warning ("Embossing is not yet supported");
 	default:
 		g_error ("Bad wallpaper type");
 		break;
@@ -1175,6 +1260,8 @@ need_wallpaper_load_p (const BGApplier *bg_applier, const BGPreferences *prefs)
 {
 	if (bg_applier->p->last_prefs == NULL)
 		return TRUE;
+	else if (prefs->wallpaper_enabled && bg_applier->p->wallpaper_pixbuf == NULL)
+		return TRUE;
 	else if (bg_applier->p->last_prefs->wallpaper_enabled != prefs->wallpaper_enabled)
 		return TRUE;
 	else if (!bg_applier->p->last_prefs->wallpaper_enabled && !prefs->wallpaper_enabled)
@@ -1190,7 +1277,13 @@ need_wallpaper_load_p (const BGApplier *bg_applier, const BGPreferences *prefs)
 static gboolean
 need_root_pixmap_p (const BGApplier *bg_applier, const BGPreferences *prefs) 
 {
-	if (bg_applier->p->last_prefs == NULL)
+	if (bg_applier->p->pixmap == NULL)
+		return TRUE;
+	else if (prefs->wallpaper_enabled == FALSE && prefs->gradient_enabled == FALSE)
+		return FALSE;
+	else if (bg_applier->p->last_prefs == NULL)
+		return TRUE;
+	else if (bg_applier->p->last_prefs->wallpaper_enabled == FALSE && bg_applier->p->last_prefs->gradient_enabled == FALSE)
 		return TRUE;
 	else if (render_small_pixmap_p (bg_applier->p->last_prefs) != render_small_pixmap_p (prefs))
 		return TRUE;
@@ -1224,7 +1317,7 @@ wallpaper_full_cover_p (const BGApplier *bg_applier, const BGPreferences *prefs)
 	else if (prefs->wallpaper_type == WPTYPE_STRETCHED)
 		return TRUE;
 
-	gdk_window_get_size (GDK_ROOT_PARENT (), &swidth, &sheight);
+	gdk_drawable_get_size (gdk_get_default_root_window (), &swidth, &sheight);
 	pwidth = gdk_pixbuf_get_width (bg_applier->p->wallpaper_pixbuf);
 	pheight = gdk_pixbuf_get_height (bg_applier->p->wallpaper_pixbuf);
 
@@ -1412,12 +1505,40 @@ is_nautilus_running (void)
 static gboolean
 cleanup_cb (BGApplier *bg_applier)
 {
-	if (bg_applier->p->wallpaper_pixbuf != NULL)
-	{
+	g_message ("cleanup_cb: Enter");
+
+	if (bg_applier->p->wallpaper_pixbuf != NULL) {
 		g_object_unref (G_OBJECT (bg_applier->p->wallpaper_pixbuf));
 		bg_applier->p->wallpaper_pixbuf = NULL;	
 	}
+
+	if (bg_applier->p->pixbuf != NULL) {
+		g_object_unref (G_OBJECT (bg_applier->p->pixbuf));
+		bg_applier->p->pixbuf = NULL;
+	}
+
 	bg_applier->p->timeout = 0;
 	
 	return FALSE;
 }
+
+static void
+preview_realized_cb (GtkWidget *preview, BGApplier *bg_applier)
+{
+	GdkPixmap *pixmap;
+
+	/* Only draw clean image if no pref set yet */
+	if (bg_applier->p->last_prefs)
+		return;
+
+	gtk_image_get_pixmap (GTK_IMAGE (preview), &pixmap, NULL);
+	
+	gdk_draw_rectangle (pixmap,
+			    preview->style->bg_gc[preview->state],
+			    TRUE,
+			    bg_applier->p->render_geom.x,
+			    bg_applier->p->render_geom.y,
+			    bg_applier->p->render_geom.width,
+			    bg_applier->p->render_geom.height);
+}
+
