@@ -18,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  *
- * Authors: Richard Hestilow <tvgm@ximian.com>
+ * Authors: Rachel Hestilow <hestilow@ximian.com>
  *
  */
 
+#include <config.h>
 #include "editor-shell.h"
 #include <gnome.h>
 #include <mtm/mtm.h>
-#include <config.h>
 #include "save.h"
 
 static gchar*
@@ -58,6 +58,13 @@ main (int argc, char **argv)
 	GtkWidget *bbox, *vbox;
 	GtkWidget *save, *close;
 
+#ifdef ENABLE_NLS
+	/* Initialize the i18n stuff */
+	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	textdomain(GETTEXT_PACKAGE);
+#endif
+
 	gnome_program_init ("metatheme-edit", VERSION, LIBGNOMEUI_MODULE,
 			    argc, argv, NULL);
 
