@@ -14,7 +14,7 @@ typedef struct _MtmEnvClass MtmEnvClass;
 
 /* This is a completely opaque type. */
 
-GtkType mtm_env_get_type (void);
+GType mtm_env_get_type (void);
 MtmEnv *mtm_env_new (void);
 const gchar *mtm_env_get_tmpdir (MtmEnv *env);
 
