@@ -63,7 +63,7 @@ struct _MtmExtHandlerClass
 	MtmHandlerClass parent_class;
 };
 
-GtkType mtm_ext_handler_get_type (void);
+GType mtm_ext_handler_get_type (void);
 
 MtmExtHandler* mtm_ext_handler_new (MtmEnv *env);
 void mtm_ext_handler_construct (MtmExtHandler *handler, MtmEnv *env);
