@@ -28,7 +28,7 @@ struct _MtmStatefulClass
 	GtkObjectClass parent_class;
 };
 
-GtkType mtm_stateful_get_type (void);
+GType mtm_stateful_get_type (void);
 
 MtmEnv *mtm_stateful_get_env (MtmStateful *stateful);
 void mtm_stateful_set_env (MtmStateful *stateful, MtmEnv *env);
