@@ -27,7 +27,7 @@ struct _PluginManagerClass
 	GtkObjectClass parent_class;
 };
 
-GtkType plugin_manager_get_type (void);
+GType plugin_manager_get_type (void);
 PluginManager* plugin_manager_new (void);
 
 gboolean plugin_manager_load (PluginManager *manager, MtmPlugin *plugin);
