@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  *
- * Authors: Richard Hestilow <hestgray@ionet.net>
+ * Authors: Rachel Hestilow <hestilow@ximian.com>
  *
  */
 
@@ -363,14 +363,28 @@ bg_ext_handler_get_config (BgExtHandler *handler)
 	BGPreferences *prefs;
 	g_return_val_if_fail (handler != NULL, NULL);
 	g_return_val_if_fail (handler->priv->ext != NULL, NULL);
-	prefs = g_object_get_data (G_OBJECT (handler->priv->ext), "bgconfig");	
+	prefs = g_object_get_data (G_OBJECT (handler->priv->ext), "bgconfig");
 	
-	if (strcmp (prefs->wallpaper_filename, handler->priv->ext->file))
+	if (prefs->wallpaper_filename)
 	{
-		g_free (prefs->wallpaper_filename);
-		prefs->wallpaper_filename = g_strdup (handler->priv->ext->file);
+		if (handler->priv->ext->file)
+		{
+			if (strcmp (prefs->wallpaper_filename, handler->priv->ext->file))
+			{
+				g_free (prefs->wallpaper_filename);
+				prefs->wallpaper_filename = g_strdup (handler->priv->ext->file);
+			}
+		}
+		else
+		{
+			g_free (prefs->wallpaper_filename);
+			prefs->wallpaper_filename = NULL;
+		}
 	}
 
+	if (!prefs->wallpaper_filename)
+		prefs->wallpaper_enabled = FALSE;
+
 	return prefs;
 }
 
