@@ -22,6 +22,7 @@
  *
  */
 
+#include <config.h>
 #include <mtm/mtm.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
@@ -153,6 +154,13 @@ mtm_init_plugin (MtmPlugin *pd)
 	
 	g_return_val_if_fail (MTM_IS_PLUGIN (pd), -1);
 
+#ifdef ENABLE_NLS
+	/* Initialize the i18n stuff */
+	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	textdomain(GETTEXT_PACKAGE);
+#endif
+
 	pd->title = g_strdup ("Nautilus Plugin");
 
 	env = MTM_STATEFUL (pd)->env;
